@@ -47,6 +47,9 @@ from .core import RosdepInternalError, InstallFailed
 from .installers import RosdepInstaller
 from .lookup import RosdepLookup, ResolutionError
 
+class UsageError(Exception):
+    pass
+
 _usage = """usage: rosdep [options] <command> <args>
 
 Commands:
@@ -77,14 +80,27 @@ rosdep where-defined <rosdeps>...
 def _get_default_RosdepLookup():
     return RosdepLookup.create_from_rospkg()
 
-def rosdep_main():
+def rosdep_main(args=None):
+    if args is None:
+        args = sys.argv[1:]
     try:
-        exit_code = _rosdep_main()
-        sys.exit(exit_code)
+        exit_code = _rosdep_main(args)
+        if exit_code not in [0, None]:
+            sys.exit(exit_code)
+    except rospkg.ResourceNotFound as e:
+        print("""
+ERROR: Rosdep cannot find all required resources to answer your query
+Missing resource: %s
+"""%(e.args[0]))
+        sys.exit(1)
+    except UsageError as e:
+        print(_usage, file=sys.stderr)
+        print("ERROR: %s"%(str(e)), file=sys.stderr)
+        sys.exit(os.EX_USAGE)
     except RosdepInternalError as e:
         print("""
 ERROR: Rosdep experienced an internal error.
-Please go to the rosdep page [1] and file a bug report with the stack trace below.
+Please go to the rosdep page [1] and file a bug report with the message below.
 [1] : http://www.ros.org/wiki/rosdep
 
 %s
@@ -107,7 +123,7 @@ Please go to the rosdep page [1] and file a bug report with the stack trace belo
 """%(e, traceback.format_exc(e)), file=sys.stderr)
         sys.exit(1)
         
-def _rosdep_main():
+def _rosdep_main(args):
     parser = OptionParser(usage=_usage, prog='rosdep')
     parser.add_option("--os", dest="os_override", default=None, 
                       metavar="OS_NAME:OS_VERSION", help="Override OS name and version (colon-separated), e.g. ubuntu:lucid")
@@ -126,7 +142,7 @@ def _rosdep_main():
     parser.add_option("-a", "--all", dest="rosdep_all", default=False, 
                       action="store_true", help="select all packages")
 
-    options, args = parser.parse_args()
+    options, args = parser.parse_args(args)
 
     if len(args) == 0:
         parser.error("Please enter a command")
@@ -172,7 +188,7 @@ def _package_args_handler(command, parser, options, args, rospack=None, rosstack
     packages = val[0]
     not_found = val[1]
     if not_found:
-        parser.error("ERROR: the following are neither a package or stack: %s"%(' '.join(not_found)))
+        raise rospkg.ResourceNotFound(not_found[0], rospack.get_ros_paths())
 
     if not packages:
         # possible with empty stacks
@@ -184,12 +200,14 @@ def _package_args_handler(command, parser, options, args, rospack=None, rosstack
 def configure_installer_context_os(installer_context, options):
     """
     Override the OS detector in *installer_context* if necessary.
+
+    :raises: :exc:`UsageError` If user input options incorrectly
     """
     if not options.os_override:
         return
     val = options.os_override
     if not ':' in val:
-        parser.error("OS override must be colon-separated OS_NAME:OS_VERSION, e.g. ubuntu:maverick")
+        raise UsageError("OS override must be colon-separated OS_NAME:OS_VERSION, e.g. ubuntu:maverick")
     os_name = val[:val.find(':')]
     os_version = val[val.find(':')+1:]
     installer_context.set_os_override(os_name, os_version)
@@ -200,6 +218,7 @@ def command_keys(lookup, packages, options):
     for package_name in packages:
         rosdep_keys.extend(lookup.get_rosdeps(package_name, implicit=True))
 
+    _print_lookup_errors(lookup)
     print('\n'.join(set(rosdep_keys)))
 
 def command_check(lookup, packages, options):
@@ -216,12 +235,16 @@ def command_check(lookup, packages, options):
         print("System dependencies have not been satisified:")
         for installer_key, resolved in uninstalled.items():
             if resolved:
-                print("%s\t%s"%(installer_key, '\n'.join(resolved)))
+                for r in resolved:
+                    print("%s\t%s"%(installer_key, r))
     else:
         print("All system dependencies have been satisified")
     if errors:
         for package_name, ex in errors.items():
-            print("ERROR[%s]: %s"%(package_name, str(ex)), file=sys.stderr)
+            if isinstance(ex, rospkg.ResourceNotFound):
+                print("ERROR[%s]: resource not found [%s]"%(package_name, ex.args[0]), file=sys.stderr)
+            else:
+                print("ERROR[%s]: %s"%(package_name, str(ex)), file=sys.stderr)                
     if uninstalled:
         return 1
     else:
@@ -276,15 +299,20 @@ def command_depdb(lookup, packages, options):
     print(_compute_depdb_output(r, args, options))
     return 0
 
+def _print_lookup_errors(lookup):
+    for error in lookup.get_errors():
+        if isinstance(error, rospkg.ResourceNotFound):
+            print("WARNING: unable to locate resource %s"%(str(error.args[0])), file=sys.stderr)
+        else:
+            print("WARNING: %s"%(str(error)), file=sys.stderr)
+            
 def command_what_needs(args, options):
     lookup = _get_default_RosdepLookup()
     packages = []
     for rosdep_name in args:
         packages.extend(lookup.get_resources_that_need(rosdep_name))
 
-    for error in lookup.get_errors():
-        print("WARNING: %s"%(str(error)), file=sys.stderr)
-
+    _print_lookup_errors(lookup)
     print('\n'.join(set(packages)))
     
 def command_where_defined(args, options):
@@ -293,12 +321,14 @@ def command_where_defined(args, options):
     for rosdep_name in args:
         locations.extend(lookup.get_views_that_define(rosdep_name))
 
-    for error in lookup.get_errors():
-        print("WARNING: %s"%(str(error)), file=sys.stderr)
-
-    for location in locations:
-        origin = location[1]
-        print(origin)
+    _print_lookup_errors(lookup)
+    if locations:
+        for location in locations:
+            origin = location[1]
+            print(origin)
+    else:
+        print("ERROR: cannot find definition(s) for [%s]"%(', '.join(args)), file=sys.stderr)
+        sys.exit(1)
 
 def command_resolve(args, options):
     lookup = _get_default_RosdepLookup()
